<?php
/**
 * @copyright Copyright (c) 2017 Joas Schilling <coding@schilljs.com>
 *
 * @license GNU AGPL version 3 or any later version
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

namespace OCA\IndieExternal;

use OCA\IndieExternal\Exceptions\SiteNotFoundException;
use OCP\App\IAppManager;
use OCP\Files\IAppData;
use OCP\IConfig;
use OCP\IGroupManager;
use OCP\IRequest;
use OCP\IUser;
use OCP\IUserSession;
use OCP\L10N\IFactory;
use OCP\INavigationManager;
use OCP\IURLGenerator;

class SitesManager {

	const TYPE_LINK = 'link';
	const TYPE_SETTING = 'settings';
	const TYPE_LOGIN = 'guest';
	const TYPE_QUOTA = 'quota';

	const DEVICE_ALL = '';
	const DEVICE_ANDROID = 'android';
	const DEVICE_IOS = 'ios';
	const DEVICE_DESKTOP = 'desktop';
	const DEVICE_BROWSER = 'browser';

	const APP_NAME = 'indie_external';

	/** @var IRequest */
	protected $request;

	/** @var IConfig */
	protected $config;

	/** @var IFactory */
	protected $languageFactory;

	/** @var IAppManager */
	protected $appManager;

	/** @var IGroupManager */
	protected $groupManager;

	/** @var IUserSession */
	protected $userSession;

	/** @var IAppData */
	protected $appData;

	/** @var IURLGenerator */
	protected $urlGenerator;

	public function __construct(IRequest $request,
								IConfig $config,
								IAppManager $appManager,
								IGroupManager $groupManager,
								IUserSession $userSession,
								IFactory $languageFactory,
								IAppData $appData,
								IURLGenerator $urlGenerator) {
		$this->request = $request;
		$this->config = $config;
		$this->appManager = $appManager;
		$this->groupManager = $groupManager;
		$this->userSession = $userSession;
		$this->languageFactory = $languageFactory;
		$this->appData = $appData;
		$this->urlGenerator = $urlGenerator;
	}

	/**
	 * @param int $id
	 * @return array
	 * @throws SiteNotFoundException
	 */
	public function getSiteById($id) {
		$sites = $this->getSites();
		foreach ($sites as $index => $site) {
			if ($id == $site["id"]) {
				return $site;
			}
		}
		throw new SiteNotFoundException();
	}

	/**
	 * @return array[]
	 */
	public function getSites() {
		$chat_url = $this->config->getAppValue(self::APP_NAME, "chat_url");
		$sso_account_url = $this->config->getAppValue(self::APP_NAME, "sso_account_url");
		$sso_admin_url = $this->config->getAppValue(self::APP_NAME, "sso_admin_url");
		$sso_redirect = filter_var($this->config->getAppValue(self::APP_NAME, "sso_redirect"), FILTER_VALIDATE_BOOLEAN); 
		$visio_url = $this->config->getAppValue(self::APP_NAME, "visio_url");

		$user = $this->userSession->getUser();
		$isAdmin = false;
		if ($user instanceof IUser) {
			$isAdmin = $this->groupManager->isAdmin($user->getUID());
		}

		$sites = [];

		if ($chat_url) {
			$sites[] = [
				'id' => 1,
				'name' => "Chat",
				'icon' =>  "conversation.png",
				'url' => $chat_url,
				'type' => INavigationManager::TYPE_APPS,
				'redirect' => true,
			];
		}


		if ($sso_admin_url && $isAdmin) {
			$sites[] = [
				'id' => 2,
				'name' => "Admin SSO",
				'icon' =>  "users.png",
				'url' => $sso_admin_url,
				'type' => INavigationManager::TYPE_SETTINGS,
				'redirect' => $sso_redirect,
			];
		}

		if ($sso_account_url) {
			$sites[] = [
				'id' => 3,
				'name' => "Mon compte Liiibre",
				'icon' =>  "compte.png",
				'url' => $sso_account_url,
				'type' => INavigationManager::TYPE_SETTINGS,
				'redirect' => $sso_redirect,
			];
		}

		$sites[] = [
			'id' => 4,
			'name' => "Centre de Documentation",
			'icon' =>  "lifesaver.png",
			'url' => "https://support.indie.host/help/",
			'type' => INavigationManager::TYPE_SETTINGS,
			'redirect' => true,
		];

		if ($visio_url) {
			$sites[] = [
				'id' => 5,
				'name' => "Visio",
				'icon' =>  "meet.png",
				'url' => $visio_url,
				'type' => INavigationManager::TYPE_APPS,
				'redirect' => true,
			];
		}


		if ($isAdmin) {
			$sites[] = [
				'id' => 6,
				'name' => "Quota des comptes",
				'icon' =>  "quota.png",
				'url' => "/settings/users",
				'type' => INavigationManager::TYPE_SETTINGS,
				'redirect' => true,
			];
		}

		return $sites;
	}

	/**
	 * @return array[]
	 */
	public function getNavigationEntries() {
		$sites = $this->getSites();
		$entries = [];
		foreach ($sites as $id => $site) {
			$image = $this->urlGenerator->linkToRoute('indie_external.icon.showIcon', ['icon' => $site['icon']]);
			$href = $site['url'];
			if (!$site['redirect']) {
				$href = $this->urlGenerator->linkToRoute('indie_external.site.showPage', ['id' => $site['id']]);
			}

			$entries[] = [
				'id' => 'indie_external_index' . $site['id'],
				'order' => 80 + $site['id'],
				'href' => $href,
				'icon' => $image,
				'type' => $site['type'],
				'name' => $site['name'],
			];
		}

		return $entries;
	}
}
